package com.ruoyi.system.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EBrandMapper;
import com.ruoyi.system.domain.EBrand;
import com.ruoyi.system.service.IEBrandService;

/**
 * 品牌商Service业务层处理
 * 
 * @author 邱贞招
 * @date 2024-07-31
 */
@Service
public class EBrandServiceImpl implements IEBrandService 
{
    @Autowired
    private EBrandMapper eBrandMapper;

    /**
     * 查询品牌商
     * 
     * @param brandId 品牌商主键
     * @return 品牌商
     */
    @Override
    public EBrand selectEBrandByBrandId(Long brandId)
    {
        return eBrandMapper.selectEBrandByBrandId(brandId);
    }

    /**
     * 查询品牌商列表
     * 
     * @param eBrand 品牌商
     * @return 品牌商
     */
    @Override
    public List<EBrand> selectEBrandList(EBrand eBrand)
    {
        return eBrandMapper.selectEBrandList(eBrand);
    }

    /**
     * 新增品牌商
     * 
     * @param eBrand 品牌商
     * @return 结果
     */
    @Override
    public int insertEBrand(EBrand eBrand)
    {
        eBrand.setCreateTime(DateUtils.getNowDate());
        return eBrandMapper.insertEBrand(eBrand);
    }

    /**
     * 修改品牌商
     * 
     * @param eBrand 品牌商
     * @return 结果
     */
    @Override
    public int updateEBrand(EBrand eBrand)
    {
        eBrand.setUpdateTime(DateUtils.getNowDate());
        return eBrandMapper.updateEBrand(eBrand);
    }

    /**
     * 批量删除品牌商
     * 
     * @param brandIds 需要删除的品牌商主键
     * @return 结果
     */
    @Override
    public int deleteEBrandByBrandIds(Long[] brandIds)
    {
        return eBrandMapper.deleteEBrandByBrandIds(brandIds);
    }

    /**
     * 删除品牌商信息
     * 
     * @param brandId 品牌商主键
     * @return 结果
     */
    @Override
    public int deleteEBrandByBrandId(Long brandId)
    {
        return eBrandMapper.deleteEBrandByBrandId(brandId);
    }
}
