package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.EUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.common.utils.wx.vo.PrepayResponseVO;
import com.ruoyi.system.domain.device.EDevice;
import com.ruoyi.system.domain.device.EDeviceQuery;
import com.ruoyi.system.domain.model.EModelVO;
import com.ruoyi.system.domain.order.EOrder;
import com.ruoyi.system.domain.order.EOrderQuery;
import com.ruoyi.system.domain.order.EOrderVO;
import com.ruoyi.system.domain.orderOper.EOrderOper;
import com.ruoyi.system.domain.refund.ERefund;
import com.ruoyi.system.domain.refund.ERefundVO;
import com.ruoyi.system.domain.rule.EFeeRule;
import com.ruoyi.system.domain.vo.IndexAdminVo;
import com.ruoyi.system.domain.vo.PriceVO;
import com.ruoyi.system.mapper.EOrderMapper;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;

/**
 * 订单Service业务层处理
 *
 * @author qzz
 * @date 2024-09-19
 */
@Service
public class EOrderServiceImpl implements IEOrderService
{

    private static final Logger logger = LoggerFactory.getLogger(EOrderServiceImpl.class);

    @Resource
    private EOrderMapper orderMapper;

    @Resource
    private IWxPayService wxPayService;

    @Autowired
    private IEFeeRuleService feeRuleService;

//    @Resource
//    private IRlCityService cityService;

//    @Autowired
//    private IRlAccessoryService accessoryService;

    @Autowired
    private IEDeviceService deviceService;

//    @Resource
//    private IRlAgentService agentService;

//    @Autowired
//    private IRlModelService modelService;

    @Autowired
    private IEModelService modelService;

    @Autowired
    private ISysConfigService configService;
//
//    @Autowired
//    private RlStoreService storeService;

    @Resource
    private IEUserService userService;

    @Autowired
    private TransactionTemplate transactionTemplate;

//    @Autowired
//    private IRlUserExtService userExtService;

    @Autowired
    private IERefundService refundService;

    @Autowired
    private IEOrderOperService rlOrderOperService;

//    @Resource
//    private IRlDividendDetailService rlDividendDetailService;

    @Autowired
    private IEOrderOperService orderOperService;

    @Autowired
    private IEChangeBalanceService changeBalanceService;

//    @Resource
//    private IRlDividendDetailService dividendDetailService;
//
//    @Autowired
//    private IRlDeliveryOrderService deliveryOrderService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private IEMsgLogService msgLogService;

    @Value("${aliyun.accessKeyId}")
    private String accessKeyId;

    @Value("${aliyun.accessKeySecret}")
    private String accessKeySecret;

    @Value("${aliyun.signName}")
    private String signName;

    @Value("${aliyun.templateCode3}")
    private String templateCode3;

    /**
     * 查询订单
     *
     * @param orderId 订单主键
     * @return 订单
     */
    @Override
    public EOrderVO selectRlOrderByOrderId(Long orderId)
    {
        return orderMapper.selectRlOrderByOrderId(orderId);
    }

    /**
     * 查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    @Override
    public EOrderVO selectRlOrderByOrderNo(String orderNo)
    {
        return orderMapper.selectRlOrderByOrderNo(orderNo);
    }

    /**
     * 第三方订单号
     *
     * @param outTradeNo 订单号
     * @return 订单
     */
    @Override
    public EOrderVO selectRlOrderByOutTradeNo(String outTradeNo)
    {
        return orderMapper.selectRlOrderByOutTradeNo(outTradeNo);
    }


    /**
     * 查询订单
     *
     * @param orderNo 订单号
     * @return 订单
     */
    @Override
    public EOrderVO getOrderInfoByOrderNo(String orderNo)
    {
        EOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(ObjectUtil.isNull(order)){
            throw new ServiceException("订单不存在");
        }
        if(order.getModelId() == null){
           throw new ServiceException("订单中没有车型id");
        }
//        RlModelVO rlModelVO = modelService.selectEModelByModelId(order.getModelId());
//        if(ObjectUtil.isNotNull(rlModelVO)){
//            List<RlFunction> functionList = modelService.getFunctionListByModelId(order.getModelId());
//            List<RlAccessoryVO> rlAccessoryVOS = accessoryService.selectRlAccessoryListByModelId(order.getModelId());
//            order.setFunctionList(functionList);
//            order.setAccessorys(rlAccessoryVOS);
//        }

        EDevice eDevice = deviceService.selectEDeviceBySn(order.getSn());
        if(ObjectUtil.isNotNull(eDevice)) {
            order.setPicture(eDevice.getPicture());
        }
        /** 超时规则*/
        Long ruleId = order.getRuleId();
        EFeeRule feeRule = feeRuleService.selectRlFeeRuleByRuleId(ruleId);
        order.setRule(feeRule);

        /** 操作记录*/
        EOrderOper orderOper = new EOrderOper();
        orderOper.setOrderNo(orderNo);
        List<EOrderOper> rlOrderOpers = rlOrderOperService.selectRlOrderOperList(orderOper);
        order.setOrderOpers(rlOrderOpers);
        /** 收益明细*/
//        List<RlDividendDetailVO> rlDividendDetails = rlDividendDetailService.selectRlDividendDetailListByOrderNo(orderNo);
//        order.setRlDividendDetails(rlDividendDetails);
//
//        RlDeliveryOrder deliveryOrder = deliveryOrderService.selectRlDeliveryOrderByOrderNo(orderNo);
//        if(ObjectUtil.isNotNull(deliveryOrder)){
//            order.setDeliveryStatus(deliveryOrder.getStatus());
//            order.setDeliveryOrder(deliveryOrder);
//        }

        /** 显示可退款金额  根据订单号查询可退款金额  refundableAmount*/
        getRefundable(orderNo, order);
        return order;
    }

    private void getRefundable(String orderNo, EOrderVO order) {
        ERefundVO refundVO = refundService.getRefundedAmount(orderNo);
        BigDecimal refundableAmount = order.getPayFee().subtract(refundVO.getTotalRefundAmount());
        BigDecimal refundableDispatchFee = order.getDispatchFee().subtract(refundVO.getTotalDispatchFee());
        BigDecimal refundableDeliveryFee = order.getDeliveryFee().subtract(refundVO.getTotalDeliveryFee());
        BigDecimal refundableDeposit = order.getDeposit().subtract(refundVO.getTotalDeposit());
        BigDecimal refundableLeaseFee = order.getLeaseFee().subtract(refundVO.getTotalLeaseFee());
        order.setRefundableTotalAmount(refundableAmount);
        order.setRefundableDispatchFee(refundableDispatchFee);
        order.setRefundableDeliveryFee(refundableDeliveryFee);
        order.setRefundableDeposit(refundableDeposit);
        order.setRefundableLeaseFee(refundableLeaseFee);
    }

    /**
     * 查询订单列表
     *
     * @param rlOrder 订单
     * @return 订单
     */
    @Override
    @DataScope(merchantAlias = "m", userAlias = "u")
    public List<EOrderVO> selectRlOrderList(EOrderQuery rlOrder)
    {
        return orderMapper.selectRlOrderList(rlOrder);
    }

    /**
     * 查询订单列表  不待数据隔离
     *
     * @param rlOrder 订单
     * @return 订单
     */
    @Override
    public List<EOrderVO> selectRlOrderListNoIsolate(EOrderQuery rlOrder)
    {
        return orderMapper.selectRlOrderList(rlOrder);
    }

    /**
     * 查询所有订单列表
     *
     * @param rlOrder 订单
     * @return 订单
     */
    @Override
    public List<EOrderVO> selectAllRlOrderList(EOrderQuery rlOrder)
    {
        return orderMapper.selectAllRlOrderList(rlOrder);
    }

    /**
     * 新增订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    @Override
    public int insertRlOrder(EOrderQuery rlOrder)
    {
        rlOrder.setCreateTime(DateUtils.getNowDate());
        return orderMapper.insertRlOrder(rlOrder);
    }

    /**
     * 修改订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    @Override
    public int updateRlOrder(EOrder rlOrder)
    {
        return orderMapper.updateRlOrder(rlOrder);
    }

    /**
     * 根据订单号修改订单
     *
     * @param rlOrder 订单
     * @return 结果
     */
    @Override
    public int updateRlOrderByOrderNo(EOrder rlOrder)
    {
        return orderMapper.updateRlOrderByOrderNo(rlOrder);
    }


    /**
     * 批量删除订单
     *
     * @param orderIds 需要删除的订单主键
     * @return 结果
     */
    @Override
    public int deleteRlOrderByOrderIds(Long[] orderIds)
    {
        return orderMapper.deleteRlOrderByOrderIds(orderIds);
    }

    /**
     * 删除订单信息
     *
     * @param orderId 订单主键
     * @return 结果
     */
    @Override
    public int deleteRlOrderByOrderId(Long orderId)
    {
        return orderMapper.deleteRlOrderByOrderId(orderId);
    }

    /**
     * 下单
     * 1. 创建订单
     * 2. 预支付
     * 3.
     *
     * @param order 订单对象
     * @return 结果
     */
    @Override
    public PrepayResponseVO createOrder(EOrderQuery order, Long userId) {
        String orderNo = IdUtils.getOrderNo("zl");
        PrepayResponseVO responseVO = getPrepayResponseVO(order,orderNo,ServiceConstants.ORDER_TYPE_LEASE,userId);
        return responseVO;
    }

    /**
     *  确认支付
     * @param orderNo 订单对象
     * @return 结果
     */
    @Override
    public PrepayResponseVO confirmPay(String orderNo) {
        EOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order.getPaid().equals(ServiceConstants.ORDER_PAY_STATUS_PAID)){
            throw new ServiceException("订单已支付");
        }
        if(!order.getStatus().equals(ServiceConstants.ORDER_STATUS_TO_BE_PAID)){
            throw new ServiceException("订单非待支付状态");
        }
        PrepayWithRequestPaymentResponse response = wxPayService.prepayWithRequestPayment(order);
        PrepayResponseVO responseVO = new PrepayResponseVO();
        responseVO.setResponse(response);
        responseVO.setOrderNo(orderNo);
        return responseVO;
    }

    @NotNull
    private PrepayResponseVO getPrepayResponseVO(EOrderQuery order, String orderNo, String type, Long userId) {
        EFeeRule feeRule = feeRuleService.selectRlFeeRuleByRuleId(order.getRuleId());
        setFeeRule(type,order, feeRule);
        /** 解析地址  analyzeAddress */
//        analyzeAddress(order);
        order.setCreateTime(DateUtils.getNowDate());
        order.setOrderNo(orderNo);
        order.setPaid(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT);
        order.setType(type);
        order.setUserId(userId);
        EUser rlUser = userService.selectUserById(userId);
        if(ObjectUtil.isNotNull(rlUser)){
            order.setUserName(rlUser.getUserName());
            order.setPhone(rlUser.getPhonenumber());
            order.setRealName(rlUser.getRealName());
        }
        String configByKey = configService.selectConfigByKey("rl.order.auto.cancel");
        order.setAutoCancelTime(DateUtils.getTimeAfterXMinutes(order.getCreateTime(),Integer.parseInt(configByKey)));

        /** 计算价格 */
        EDevice eDevice = deviceService.selectEDeviceBySn(order.getSn());
        if(eDevice == null){
            throw new ServiceException("设备不存在");
        }
        if(eDevice.getModelId() == null){
            throw new ServiceException("设备sn【"+order.getSn()+"】的车型不存在");
        }
        order.setModelId(eDevice.getModelId());
        onceMoreCalculatePrice(order);
        order.setStatus(ServiceConstants.ORDER_STATUS_TO_BE_PAID);
        EModelVO rlModelVO = modelService.selectEModelByModelId(order.getModelId());
        if(ObjectUtil.isNotNull(rlModelVO)){
            order.setModel(rlModelVO.getModel());
        }
        //先计算平台服务费，再将扣除掉平台服务费订单金额拿去分成
        if(ObjectUtil.isNull(rlUser.getServiceFeeProportion())){
            throw new ServiceException("用户【"+rlUser.getUserName()+"】的平台服务费未配置");
        }
        BigDecimal serviceFee = rlUser.getServiceFeeProportion()
                .multiply(order.getPayFee())
                .setScale(2, RoundingMode.HALF_UP);// 服务费 = 48 * 0.05 = 2.4
        order.setPlatformServiceFee(serviceFee);
        logger.info("计算出平台服务费：{}",serviceFee);

        PrepayResponseVO responseVO = new PrepayResponseVO();
        Boolean execute = transactionTemplate.execute(e -> {
            logger.info("=================【下单】==================");
            /** 保存订单*/
            int i = orderMapper.insertRlOrder(order);
            if (i < 1) {
                throw new RuntimeException("下单失败");
            }
            /** 支付参数 */
            PrepayWithRequestPaymentResponse response = wxPayService.prepayWithRequestPayment(order);
            responseVO.setResponse(response);
            responseVO.setOrderNo(orderNo);

            /** 发短信给代理商*/
//            YPMsgUtils.sendPlaceOrderMsg(orderNo,rlAgentVO.getPhone(),order.getPhone());
            return Boolean.TRUE;
        });
        if(Boolean.FALSE.equals(execute))throw new ServiceException("【下单】失败");
        return responseVO;
    }

    private void onceMoreCalculatePrice(EOrderQuery order) {
        PriceVO priceVO = calculatePrice(order);
        order.setDeposit(priceVO.getDeposit());
        order.setLeaseFee(priceVO.getRent());
        order.setTotalFee(priceVO.getTotalFee());
        order.setPayFee(priceVO.getTotalFee());
    }

    /** 解析地址 */
//    private void analyzeAddress(RlOrderQuery order) {
//        String address = cityService.getAddressByLocation(order.getPickupLon(),order.getPickupLat());
//        order.setPickupLoc(address);
//    }

    private void setFeeRule(String type, EOrderQuery order, EFeeRule feeRule) {
        order.setNum(order.getNum());
        order.setPrice(feeRule.getPrice());
        order.setExplain(feeRule.getExplain());
        order.setInstructions(feeRule.getInstructions());
        order.setOutUnit(feeRule.getOutUnit());
        order.setOutPrice(feeRule.getOutPrice());
        order.setRentalUnit(feeRule.getRentalUnit());
        /** 如果是租赁订单，根据取车时间计算到期时间
         * 如果是续费订单，根据到期时间计算续费订单的到期时间 */
        Date startTime = null;
        if(ServiceConstants.ORDER_TYPE_LEASE.equals(type)){
            startTime = DateUtils.getNowDate();
        }else{
            startTime = order.getExpiryTime();
        }
        Date expiryTime = DateUtils.getTimeAfterXUnit(startTime, order.getNum(), feeRule.getRentalUnit());//先根据预估的取车时间，计算出到期时间，后面根据具体的取车时间再算一次
        order.setExpiryTime(expiryTime);
    }

    /**
     * 扫码绑定车辆
     */
    @Override
    public int bandDevice(String orderNo, String sn) {
        EOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order == null){
            throw new RuntimeException("订单不存在");
        }
        if(StrUtil.isNotBlank(order.getSn())){
            throw new RuntimeException("该订单已绑定车辆："+order.getSn());
        }
        List<EOrderVO> order1 = getInProgressOrder(null,sn);
        if(!order1.isEmpty()){
            throw new RuntimeException("该车辆【"+sn+"】有进行中的订单：");
        }
        Boolean execute = transactionTemplate.execute(e -> {
            logger.info("=================【扫码绑定车辆】==================");
            EOrder rlOrder = new EOrder();
            rlOrder.setOrderNo(orderNo);
            rlOrder.setSn(sn);
            rlOrder.setStatus(ServiceConstants.ORDER_STATUS_IN_USE);
            rlOrder.setPickupTime(DateUtils.getNowDate());
            Date expiryTime = DateUtils.getTimeAfterXUnit(DateUtils.getNowDate(), order.getNum(), order.getRentalUnit());
            rlOrder.setExpiryTime(expiryTime);
            int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
            if(i > 0){
                /** 更新设备状态 */
                EDeviceQuery rlDevice = new EDeviceQuery();
                rlDevice.setSn(sn);
                rlDevice.setStatus(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK);
                deviceService.updateEDeviceBySn(rlDevice);
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("扫码绑定车辆失败");
        return 1;
    }

    /**
     * 用户扫码绑定车辆
     *  1. 校验sn是否存在
     *  2. 绑定车辆,在车辆中添加用户id
     */
    @Override
    public int userBandDevice(Long userId, String sn) {
        EDevice device = deviceService.selectEDeviceBySn(sn);
        if(device == null){
            throw new ServiceException("设备不存在:"+sn);
        }
        if(device.getUserId() != null && device.getUserId() != 0){
            throw new ServiceException("该车辆已绑定用户");
        }
        Boolean execute = transactionTemplate.execute(e -> {
            logger.info("=================【用户扫码绑定车辆】==================");
            EDeviceQuery rlDevice = new EDeviceQuery();
            rlDevice.setSn(sn);
            rlDevice.setUserId(userId);
            deviceService.updateEDeviceBySn(rlDevice);
            return Boolean.TRUE;
        });
        if(Boolean.FALSE.equals(execute))throw new ServiceException("【用户扫码绑定车辆】失败");
        return 1;
    }

//        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
//        if(order == null){
//            throw new RuntimeException("订单不存在");
//        }
//        if(StrUtil.isNotBlank(order.getSn())){
//            throw new RuntimeException("该订单已绑定车辆："+order.getSn());
//        }
//        RlOrderVO order1 = getInProgressOrder(sn);
//        if(ObjectUtil.isNotNull(order1)){
//            throw new RuntimeException("该车辆【"+sn+"】有进行中的订单：");
//        }
//        Boolean execute = transactionTemplate.execute(e -> {
//            logger.info("=================【扫码绑定车辆】==================");
//            RlOrder rlOrder = new RlOrder();
//            rlOrder.setOrderNo(orderNo);
//            rlOrder.setSn(sn);
//            rlOrder.setStatus(ServiceConstants.ORDER_STATUS_IN_USE);
//            rlOrder.setPickupTime(DateUtils.getNowDate());
//            Date expiryTime = DateUtils.getTimeAfterXUnit(DateUtils.getNowDate(), order.getNum(), order.getRentalUnit());
//            rlOrder.setExpiryTime(expiryTime);
//            int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
//            if(i > 0){
//                /** 更新设备状态 */
//                EDevice rlDevice = new EDevice();
//                rlDevice.setSn(sn);
//                rlDevice.setStatus(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK);
//                deviceService.updateEDeviceBySn(rlDevice);
//            }
//            return Boolean.TRUE;
//        });
//        if(!execute)throw new ServiceException("扫码绑定车辆失败");
//        return 1;
//    }

    /**
     * 续租
     * 到期时间
     * 1.  根据原来的订单号，写入新的订单
     * 2.  续租订单标记类型
     * 3.
     */
    @Override
    public PrepayResponseVO relet(EOrderQuery query) {
        EOrderVO order = orderMapper.selectRlOrderByOrderNo(query.getOrderNo());
        if(ObjectUtil.isNull(order)){
            throw new RuntimeException("订单【"+query.getOrderNo()+"】不存在");
        }
        EOrderQuery orderQuery = new EOrderQuery();
        BeanUtils.copyBeanProp(orderQuery,order);
        /** 原订单已经到期  续租订单的到期时间是根据原订单的到期时间算出来的 */
        Date nowDate = DateUtils.getNowDate();
        if (order.getExpiryTime().before(nowDate)) {
            /** 到期后先计算逾期费用，保存在订单中，在还车时一起结算 */
            EOrder updateOrder = new EOrder();
            updateOrder.setOrderId(order.getOrderId());
            BigDecimal overdueFee= BigDecimal.ZERO;
            if(ObjectUtil.isNull(order.getExpiryTime())){
                throw new ServiceException("【续租】订单没有到期时间");
            }
            if (order.getExpiryTime().before(DateUtils.getNowDate())) {
                overdueFee = deviceService.computeOverdueFee(order, updateOrder);
            }
            updateOrder.setOverdueFee(overdueFee);
            int i = updateRlOrder(updateOrder);
            if(i==0){
                throw new ServiceException("更新订单状态失败");
            }
        }
        orderQuery.setOriginalOrderNo(query.getOrderNo());
        orderQuery.setRuleId(query.getRuleId());
        orderQuery.setPickupTime(order.getExpiryTime());
        orderQuery.setNum(query.getNum());
        String newOrderNo = IdUtils.getOrderNo("xz");
        PrepayResponseVO responseVO = getPrepayResponseVO(orderQuery,newOrderNo,ServiceConstants.ORDER_TYPE_RELET,SecurityUtils.getUserId());
        return responseVO;
    }

    /**
     * 订单取消
     *
     */
    @Override
    public Boolean orderCancel(String orderNo) {
        if(StrUtil.isBlank(orderNo)){
            throw new RuntimeException("订单号不能为空");
        }
        EOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(order == null){
            throw new RuntimeException("订单不存在");
        }
        // 取消订单改成配送工单还没有接单前可以取消订单
        // 判断是否在配送工单还没有接单前
//        if(Integer.parseInt(order.getStatus()) >= Integer.parseInt(ServiceConstants.ORDER_STATUS_IN_DELIVERY)){
//            throw new RuntimeException("订单已配送或已使用，不能取消");
//        }
        Boolean execute = transactionTemplate.execute(e -> {
            logger.info("=================【订单取消】==================");
            if(ServiceConstants.ORDER_PAY_STATUS_PAID.equals(order.getPaid())){
                // 如果订单已支付则发起退款
                EOrderQuery rlOrderQuery = new EOrderQuery();
                BeanUtils.copyBeanProp(rlOrderQuery,order);
                int refund = refund(rlOrderQuery);
                if(refund <= 0){
                    throw new RuntimeException("【订单取消】退款失败");
                }
            }
            EOrder rlOrder = new EOrder();
            rlOrder.setOrderNo(orderNo);
            rlOrder.setStatus(ServiceConstants.ORDER_STATUS_CANCE);
            int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
            if(i>0){
                /** 记录订单履历 */
                if(!orderOperService.recordOrderHistory(orderNo,ServiceConstants.ORDER_OPERATION_USER_CANCEL,
                        order.getStatus(),rlOrder.getStatus(),order.getPayFee(),order.getPayFee(),order.getUserId(),order.getPhone(),"取消订单")){
                    throw new ServiceException("【订单履历-取消订单】更新订单信息失败");
                }
                return true;
            }
            return Boolean.TRUE;
        });
        if(Boolean.FALSE.equals(execute))throw new ServiceException("订单取消失败");
        return true;
    }

    /**
     * 车损扣款
     */
    @Override
    public Boolean deductMoney(BigDecimal money, String orderNo) {
        EOrder rlOrder = new EOrder();
        rlOrder.setOrderNo(orderNo);
        rlOrder.setDeductionAmount(money);
        int i = orderMapper.updateRlOrderByOrderNo(rlOrder);
        /** 记录订单履历 */
        EOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(!orderOperService.recordOrderHistory(orderNo,ServiceConstants.ORDER_OPERATION_DEDUCTION,
                order.getStatus(),order.getStatus(),order.getPayFee(),order.getPayFee(),order.getUserId(),order.getPhone(),"车损扣款：扣款金额："+money+"，实际押金退款："+order.getDeposit()+"-"+money)){
            throw new ServiceException("【车损扣款】更新订单信息失败");
        }
        return i>0;
    }

    /**
     * 商家还车
     *
     */
    @Override
    public Boolean merchantReturn(String orderNo) {
        EOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
        if(ObjectUtil.isNull(order)){
            throw new ServiceException("订单不存在");
        }
        BigDecimal deposit = order.getDeposit();
        BigDecimal overdueFee = order.getOverdueFee();
        BigDecimal deductionAmount = order.getDeductionAmount();
        BigDecimal remainingDeposit = deposit.subtract(overdueFee).subtract(deductionAmount);
        if(!ServiceConstants.ORDER_PAY_STATUS_PAID.equals(order.getPaid())){
          throw new ServiceException("订单未支付，不能还车", HttpStatus.ERROR);
        }
        if(!(ServiceConstants.ORDER_STATUS_TO_BE_AUDITED.equals(order.getStatus()) || ServiceConstants.ORDER_STATUS_IN_USE.equals(order.getStatus()))){
            throw new ServiceException("订单非待审核或骑行中状态，不能还车", HttpStatus.ERROR);
        }
        Boolean execute = transactionTemplate.execute(e -> {
            /** 1.更新订单信息*/
            updateOrderStatus(orderNo);
            /** 2.更新车辆状态*/
            updateDeviceInfo(order.getSn());
            /** 3.记录订单履历*/
            if(!orderOperService.recordOrderHistory(orderNo,ServiceConstants.ORDER_OPERATION_RETURN_END,
                    order.getStatus(),order.getStatus(),order.getPayFee(),order.getPayFee(),order.getUserId(),order.getPhone(),"已退款：-"+remainingDeposit+"元")){
                throw new ServiceException("【记录订单履历失败】");
            }
            if(remainingDeposit.compareTo(BigDecimal.ZERO) > 0){
                /** 4.执行退还剩余押金操作*/
                /** 5.新增一条退款记录*/
                String outRefundNo = saveDepositRefundObj(order, remainingDeposit);
                logger.info("【商家还车】退还剩余押金："+remainingDeposit);
                wxPayService.refund(order, "审核后退押金",remainingDeposit,outRefundNo);
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("商家还车失败");
        return true;
    }

    @NotNull
    private String saveDepositRefundObj(EOrderVO rlOrderVO, BigDecimal refundAmount) {
        String outRefundNo = IdUtils.getOrderNo("ref");
        ERefund refund1= createRefund("押金退款", rlOrderVO.getUserId(), rlOrderVO.getOrderNo(), refundAmount, BigDecimal.ZERO, BigDecimal.ZERO, BigDecimal.ZERO, outRefundNo,ServiceConstants.REFUND_TYPE_DEPOSIT);
        int refundResult = refundService.insertEtRefund(refund1);
        if(refundResult>0){
            logger.info("保存退款对象成功");
        }
        logger.info("总押金：【{}】，退款金额：【{}】", rlOrderVO.getDeposit(), refundAmount);
        return outRefundNo;
    }

    private void updateOrderStatus(String orderNo) {
        EOrder updateOrder = new EOrder();
        updateOrder.setOrderNo(orderNo);
        updateOrder.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
        int i = orderMapper.updateRlOrderByOrderNo(updateOrder);
        if(i==0){
            logger.info("【商家还车】更新订单信息失败");
            throw new ServiceException("【商家还车】更新订单信息失败");
        }
    }

    private void updateDeviceInfo(String sn) {
        EDeviceQuery updateDevice = new EDeviceQuery();
        updateDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);
        updateDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
        updateDevice.setSn(sn);
        int deviceUpdate = deviceService.updateEDeviceBySn(updateDevice);
        if(deviceUpdate==0){
            logger.info("【商家还车】更新车辆状态失败");
            throw new ServiceException("【商家还车】更新车辆状态失败");
        }
    }

    /**
     * 退款
     * 1. 校验
     * 2. 用户余额不足判断
     * 3. 计算出退款的比例
     * 4. 遍历用户，新增账变
     * 5. 新增退款明细
     * 6. 发起退款
     */
    @Transactional
    @Override
    public int refund(EOrderQuery etOrder) {
        //根据订单号查询订单信息
        /** 1. 校验 */
        EOrderVO rlOrderVO = orderMapper.selectRlOrderByOrderNo(etOrder.getOrderNo());
        refundVerify(rlOrderVO);
        /** 2.计算出退款的比例 保留两位小数点  比如退款金额为5元，  订单金额为32元  退款比例 = 5/32 = 0.16 （四舍五入）由系统来平账*/
        //退款金额  调度费(商家取车)  配送费（送车上门）  租赁费
        BigDecimal refundAmount = new BigDecimal("0");
        BigDecimal dispatchFee = etOrder.getDispatchFee();
        if(ObjectUtil.isNotNull(dispatchFee) && !dispatchFee.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(dispatchFee);
        }
        BigDecimal deliveryFee = etOrder.getDeliveryFee();
        if(ObjectUtil.isNotNull(deliveryFee) && !deliveryFee.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(deliveryFee);
        }
        BigDecimal leaseFee = etOrder.getLeaseFee();
        if(ObjectUtil.isNotNull(leaseFee) && !leaseFee.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(leaseFee);
        }
        BigDecimal deposit = etOrder.getDeposit();
        if(ObjectUtil.isNotNull(deposit) && !deposit.equals(BigDecimal.ZERO)){
            refundAmount = refundAmount.add(deposit);
        }
        if(refundAmount.compareTo(BigDecimal.ZERO) == 0){
            throw new ServiceException("总退款金额不能为0");
        }
        BigDecimal refundPercentage = refundAmount.divide(rlOrderVO.getPayFee(), 2, RoundingMode.HALF_UP);
        BigDecimal finalRefundAmount = refundAmount;
        Boolean execute = transactionTemplate.execute(e -> {
            /** 3. 根据分成列表，遍历用户，新增账变 */
//            refundDividendHandle(rlOrderVO, refundPercentage);
            /** 4.记录退款表  创建退款对象*/
            String outRefundNo = saveRefundObj(rlOrderVO, finalRefundAmount, dispatchFee, deliveryFee, leaseFee);
            if(!orderOperService.recordOrderHistory(rlOrderVO.getOrderNo(),ServiceConstants.ORDER_OPERATION_REFUND,
                    rlOrderVO.getStatus(),rlOrderVO.getStatus(),rlOrderVO.getPayFee(),rlOrderVO.getPayFee(),rlOrderVO.getUserId(),rlOrderVO.getPhone(),"已退款: "+finalRefundAmount+"元")){
                throw new ServiceException("【退款】更新订单信息失败");
            }
            /** 5. 发起退款 */
            wxPayService.refund(rlOrderVO, "商家退款金额", finalRefundAmount,outRefundNo);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("【订单退款】失败");
        return 1;
    }

    @NotNull
    private String saveRefundObj(EOrderVO rlOrderVO, BigDecimal refundAmount, BigDecimal dispatchFee, BigDecimal deliveryFee, BigDecimal leaseFee) {
        String outRefundNo = IdUtils.getOrderNo("ref");
        ERefund refund1= createRefund("商家退款金额", rlOrderVO.getUserId(), rlOrderVO.getOrderNo(), refundAmount, dispatchFee, deliveryFee, leaseFee, outRefundNo,ServiceConstants.REFUND_TYPE_SYSTEM);
        int refundResult = refundService.insertEtRefund(refund1);
        if(refundResult>0){
            logger.info("保存退款对象成功");
        }
        logger.info("总金额：【{}】，退款金额：【{}】", rlOrderVO.getTotalFee(), refundAmount);
        return outRefundNo;
    }

//    private void refundDividendHandle(RlOrderVO rlOrderVO, BigDecimal refundPercentage) {
//        List<RlDividendDetailVO> dividendDetails = rlDividendDetailService.selectRlDividendDetailListByOrderNo(rlOrderVO.getOrderNo());
//        for(RlDividendDetail detail:dividendDetails){
//            /** 4.1 余额不足判断 */
////            RlUserExt rlUserExt = userExtService.selectRlUserExtByUserId(detail.getPartnerId());
//            EUser user = userService.selectUserById(rlUserExt.getUserId());
//            // 计算出这个用户将要退款的金额
//            BigDecimal userRefundAmount = detail.getDividendAmount().multiply(refundPercentage);//用户退款金额  = 0.16 * 5元 = 0.8元  如果这个值小于余额则提示 余额不足
//            BigDecimal subtract = user.getBalance().subtract(userRefundAmount);
//            if(subtract.compareTo(BigDecimal.ZERO) <= 0){
//                throw new ServiceException("用户【"+user.getPhonenumber()+"】余额不足，不能退款");
//            }
//            /** 4.2 新增账变 */
//            int i = changeBalanceService.generateChanggeBalance(rlOrderVO.getOrderNo(), rlOrderVO.getOutTradeNo(), ServiceConstants.FLOW_TYPE_DISBURSE,
//                    ServiceConstants.ORDER_REFUND, userRefundAmount, rlUserExt.getUserId(), user.getUserName(), user.getPhonenumber(),user.getUserType());
//            if(i<=0)throw new ServiceException("用户【"+user.getPhonenumber()+"】新增账变失败");
//            /** todo 4.3 更新分成表中的退款金额 */
//            int i1 = rlDividendDetailService.updateRefundAmount(detail.getId(), userRefundAmount);
//            if(i1<=0)throw new ServiceException("【退款】更新分成表中的退款金额失败");
//        }
//    }

    /** 退款校验*/
    private void refundVerify(EOrderVO rlOrderVO) {
        if(ObjectUtil.isNull(rlOrderVO)){
            throw new ServiceException("订单不存在");
        }
        if (!ServiceConstants.ORDER_STATUS_ORDER_END.equals(rlOrderVO.getStatus())) {
            throw new ServiceException("退款失败，订单未结束");
        }
        if(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT.equals(rlOrderVO.getPaid())){
            throw new ServiceException("订单未支付，不能退款");
        }
    }

    @Override
    public ERefund createRefund(String reason, Long userId, String orderNo, BigDecimal refundAmount, BigDecimal dispatchFee, BigDecimal deliveryFee, BigDecimal leaseFee, String outRefundNo, String type) {
        ERefund etRefund = new ERefund();
        etRefund.setAmount(refundAmount);
        etRefund.setReason(reason);
        etRefund.setDispatchFee(dispatchFee);
        etRefund.setDeliveryFee(deliveryFee);
        etRefund.setLeaseFee(leaseFee);
        etRefund.setOrderNo(orderNo);
        etRefund.setUserId(userId);
        etRefund.setRefundNo(outRefundNo);
        etRefund.setType(type);
        if(type.equals(ServiceConstants.REFUND_TYPE_DEPOSIT)){
            etRefund.setItemDesc("押金退款");
        }else{
            StringBuilder itemDesc = new StringBuilder();
            if(ObjectUtil.isNotNull(deliveryFee) && !deliveryFee.equals(BigDecimal.ZERO)){
                itemDesc.append("配送费:").append(deliveryFee).append("元,");
            }
            if(ObjectUtil.isNotNull(dispatchFee) && !dispatchFee.equals(BigDecimal.ZERO)){
                itemDesc.append("调度费:").append(dispatchFee).append("元,");
            }
            if(ObjectUtil.isNotNull(leaseFee) && !leaseFee.equals(BigDecimal.ZERO)){
                itemDesc.append("租赁费:").append(leaseFee).append("元,");
            }
            etRefund.setItemDesc(itemDesc.toString());
        }
        return etRefund;
    }

    /**
     * 计算价格
     */
    @Override
    public PriceVO calculatePrice(EOrderQuery order) {
        PriceVO priceVO = new PriceVO();
        BigDecimal totalFee = BigDecimal.ZERO;
        if(order.getType().equals(ServiceConstants.ORDER_TYPE_LEASE)){
            // 押金
            BigDecimal deposit = deposit(order.getModelId(), priceVO);
            totalFee = totalFee.add(deposit);
        }
        // 租金
        BigDecimal rentFee = rentFee(order, priceVO);
        totalFee = totalFee.add(rentFee);
        priceVO.setTotalFee(totalFee);
        return priceVO;
    }

//    /**
//     * 根据订单号查询可还车的店铺
//     */
//    @Override
//    public List<StoreVo> getReturnStoreList(String orderNo,String lon,String lat) {
//        RlOrderVO order = orderMapper.selectRlOrderByOrderNo(orderNo);
//        List<StoreVo> storeVos = storeService.selectSmStoreListBYMerchantId(order.getMerchantId());
//        for (StoreVo storeVo:storeVos) {
//            BigDecimal storeLon = storeVo.getLng();
//            BigDecimal storeLat = storeVo.getLat();
//            if (storeLon != null && storeLat != null && StrUtil.isNotBlank(lon) && StrUtil.isNotBlank(lat)) {
//                double[] point1 = {storeLon.doubleValue(), storeLat.doubleValue()};
//                double[] point2 = {Double.valueOf(lon), Double.valueOf(lat)};
//                double distance = GeoUtils.haversineDistance(point1, point2);
//                BigDecimal distanceFormatted = new BigDecimal(distance).setScale(1, RoundingMode.HALF_UP);
//                storeVo.setDistance(distanceFormatted.doubleValue());
//            }
//        }
//        List<StoreVo> storeList = new ArrayList<>(storeVos);
//        Collections.sort(storeList, new Comparator<StoreVo>() {
//            @Override
//            public int compare(StoreVo o1, StoreVo o2) {
//                // 注意：由于我们想要按距离由近到远排序，因此当 o1 的距离小于 o2 的距离时，应返回 -1
//                // 当距离相等时返回 0，当 o1 的距离大于 o2 的距离时返回 1
//                return Double.compare(o1.getDistance(), o2.getDistance());
//            }
//        });
//        return storeList;
//    }

    /**
     * 根据sn查询正在进行中的订单
     */
    @Override
    public List<EOrderVO> getInProgressOrder(Long userId, String sn) {
        return orderMapper.getInProgressOrder(userId,sn);
    }

    @Override
    public void validate(EOrderQuery orderQuery) {
        if(StrUtil.isBlank(orderQuery.getOrderNo())){
            throw new ServiceException("订单号不能为空");
        }
        if(StrUtil.isBlank(orderQuery.getType())){
            throw new ServiceException("类型不能为空");
        }
        if(orderQuery.getRuleId() == null || orderQuery.getRuleId() <= 0){
            throw new ServiceException("计费规则id不能为空");
        }
        if(orderQuery.getNum() == null || orderQuery.getNum() <= 0){
            throw new ServiceException("数量不能为空或小于0");
        }
    }

    /**
     * 首页统计
     */
    @Override
    public IndexAdminVo indexStatistics(String startTime,String endTime,Long userId) {
        // 定义统一的日期格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        // 根据用户id查询代理商，根据代理商做数据权限隔离
        EUser rlUser = userService.selectUserById(userId);
        if(!ServiceConstants.USER_TYPE_MERCHANT.equals(rlUser.getUserType())){
            throw new ServiceException("您没有权限访问");
        }
        return getIndexAdminVoByAgentId(startTime, endTime, formatter,rlUser.getUserId());
    }

    @NotNull
    private IndexAdminVo getIndexAdminVoByAgentId(String startTime, String endTime, DateTimeFormatter formatter,Long merchantId) {
        // 校验输入时间格式并判断时间区间是否超过15天
        if (StrUtil.isBlank(startTime) && StrUtil.isBlank(endTime)) {
            // 获取今日订单数量和收入
            startTime = DateUtils.getYYYY_MM_DD(DateUtils.getNowDate());
            endTime = DateUtils.getYYYY_MM_DD(DateUtils.getNowDate());
        }
        // 使用LocalDate统一日期处理
        LocalDate startDate = LocalDate.parse(startTime, formatter);
        LocalDate endDate = LocalDate.parse(endTime, formatter);

        // 计算两者之间的天数差异
        long daysBetween = ChronoUnit.DAYS.between(startDate, endDate);
        if (daysBetween > 15) {
            throw new ServiceException("时间范围不能超过15天");
        }

        // 创建结果对象
        IndexAdminVo indexAdminVo = new IndexAdminVo();
        ArrayList<IndexAdminVo.IncomeVo> incomeVos = new ArrayList<>();

        // 遍历从 startDate 到 endDate 的每一天，生成订单数据
        for (LocalDate date = startDate; !date.isAfter(endDate); date = date.plusDays(1)) {
            IndexAdminVo.IncomeVo orderFee = new IndexAdminVo.IncomeVo();
            String day = date.format(formatter);
            String startDateStr = day + " "+ Constants.DATE_FORMAT_START_PEREND;
            String endDateStr = day + " " +Constants.DATE_FORMAT_END_PEREND;
            orderFee.setDay(day); // 设置每一天的日期
            BigDecimal orderFee1 = orderMapper.getOrderFee(startDateStr, endDateStr, merchantId);//订单金额
            orderFee.setOrderFee(orderFee1);
            orderFee.setOrderNum(orderMapper.getOrderNum(startDateStr, endDateStr, merchantId));//订单数量

            incomeVos.add(orderFee);
        }
        // 设置收入数据
        indexAdminVo.setIncomeVoList(incomeVos);
        // 获取今日订单数量和收入
        String todayStart = DateUtils.getYYYY_MM_DD(DateUtils.getNowDate()) + " " + Constants.DATE_FORMAT_START_PEREND;
        String todayEnd = DateUtils.getYYYY_MM_DD(DateUtils.getNowDate()) + " " + Constants.DATE_FORMAT_END_PEREND;

        // 查询今日的订单数和营收
        Integer todayOrderNum = orderMapper.getOrderNum(todayStart, todayEnd, merchantId);
        BigDecimal todayIncome = orderMapper.getOrderFee(todayStart, todayEnd, merchantId);

        // 设置今日的订单数和营收
        indexAdminVo.setTodayOrderNum(todayOrderNum);
        indexAdminVo.setTodayIncome(todayIncome);

        String yesterdayStart = DateUtils.getYYYY_MM_DD(DateUtils.addDays(DateUtils.getNowDate(), -1)) + " " + Constants.DATE_FORMAT_START_PEREND;
        String yesterdayEnd = DateUtils.getYYYY_MM_DD(DateUtils.addDays(DateUtils.getNowDate(), -1)) + " " + Constants.DATE_FORMAT_END_PEREND;
        // 查询昨日的订单数和营收
        Integer yesterdayOrderNum = orderMapper.getOrderNum(yesterdayStart, yesterdayEnd, merchantId);
        BigDecimal yesterdayIncome = orderMapper.getOrderFee(yesterdayStart, yesterdayEnd, merchantId);

        // 计算较昨日的营收和订单数
        BigDecimal comparedYesterdayIncome = todayIncome.subtract(yesterdayIncome);
        BigDecimal comparedYesterdayOrderNum = new BigDecimal(todayOrderNum - yesterdayOrderNum);

        // 设置较昨日的营收和订单数
        indexAdminVo.setComparedYesterdayIncome(comparedYesterdayIncome);
        indexAdminVo.setComparedYesterdayOrderNum(comparedYesterdayOrderNum);

        // 设置车辆统计数据
        IndexAdminVo.DeviceVO deviceVO = new IndexAdminVo.DeviceVO();
        deviceVO.setTotal(deviceService.getTatalDeviceCount(merchantId,null,null));
        deviceVO.setIdle(deviceService.getTatalDeviceCount(merchantId,ServiceConstants.VEHICLE_STATUS_NORMAL,null));
        String[] statusList = new String[] { ServiceConstants.VEHICLE_STATUS_IN_USING,ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK };
        deviceVO.setRent(deviceService.getTatalDeviceCount(merchantId,null,statusList));
        indexAdminVo.setDeviceVO(deviceVO);
        return indexAdminVo;
    }

    private BigDecimal deposit(Long modelId, PriceVO priceVO) {
        EModelVO rlModelVO = modelService.selectEModelByModelId(modelId);
        if(rlModelVO == null){
            throw new ServiceException("车型不存在");
        }
        BigDecimal deposit = rlModelVO.getDeposit();
        if(deposit == null){
            throw new ServiceException("车型【"+modelId+"未设置押金】");
        }
        priceVO.setDeposit(deposit);
        return deposit;
    }

    private BigDecimal deliveryFee(EOrderQuery orderQuery, PriceVO priceVO) {
        BigDecimal deliveryFee = BigDecimal.ZERO;
//        RlAgent agent = agentService.selectRlAgentByAgentId(orderQuery.getAgentId());
//        if(agent == null){
//            throw new ServiceException("代理商不存在："+orderQuery.getAgentId());
//        }
//        if(orderQuery.getDeliveryMethod().equals(ServiceConstants.DELIVERY_METHOD_CHARGE_DELIVERY_CAR)){
//            deliveryFee = agent.getDeliveryFee();
//            priceVO.setDeliveryFee(deliveryFee);
//        }
        return deliveryFee;
    }

    private BigDecimal rentFee(EOrderQuery order, PriceVO priceVO) {
        BigDecimal rent = BigDecimal.ZERO;
        EFeeRule feeRule = feeRuleService.selectRlFeeRuleByRuleId(order.getRuleId());
        if(feeRule == null){
            throw new ServiceException("租赁规则不存在");
        }
        Integer num = order.getNum();
        if(num != null){
            rent = feeRule.getPrice().multiply(new BigDecimal(num));
            priceVO.setRent(rent);
        }
        return rent;
    }


}
